@@ -90,9 +90,9 @@ import org.xml.sax.SAXParseException;
 /**
  * This is the base class of all standard parsers.
  *
- * @version $Id: XMLParser.java,v 1.33 2001/06/25 18:07:51 elena Exp $
+ * @version $Id: XMLParser.java,v 1.33.2.5 2001/11/13 20:48:14 neilg Exp $
  */
-public abstract class XMLParser 
+public abstract class XMLParser
     implements XMLErrorReporter, XMLDocumentHandler.DTDHandler {
 
     //
@@ -124,9 +124,11 @@ public abstract class XMLParser
         "http://xml.org/sax/features/namespaces",
         // Xerces
         "http://apache.org/xml/features/validation/schema",
+        // "http://apache.org/xml/features/schema/expose-normalized-values",
         "http://apache.org/xml/features/validation/schema-full-checking",
         "http://apache.org/xml/features/validation/dynamic",
         "http://apache.org/xml/features/validation/default-attribute-values",
+        
         "http://apache.org/xml/features/validation/validate-content-models",
         "http://apache.org/xml/features/validation/validate-datatypes",
         "http://apache.org/xml/features/validation/warn-on-duplicate-attdef",
@@ -348,27 +350,27 @@ public abstract class XMLParser
         return fEntityHandler;
     }
 
-    
+
     /**
-     * return the locale               
+     * return the locale
      *
      * @return the locale
      */
     public final Locale getfLocale() {
-        return fLocale; 
+        return fLocale;
     }
 
     /**
-     * return the XML Messages object               
+     * return the XML Messages object
      *
      * @return the parser's messages object
      */
     public final XMLMessageProvider getfgXMLMessages() {
-        return fgXMLMessages; 
+        return fgXMLMessages;
     }
 
     /**
-     * return the Implementation Messages object               
+     * return the Implementation Messages object
      *
      * @return the parser's implementation messages
      */
@@ -377,7 +379,7 @@ public abstract class XMLParser
     }
 
     /**
-     * return the Schema Messages object               
+     * return the Schema Messages object
      *
      * @return the parser's schema messages
      */
@@ -386,7 +388,7 @@ public abstract class XMLParser
     }
 
     /**
-     * return the Datatypes Messages object               
+     * return the Datatypes Messages object
      *
      * @return the parser's datatypes messages
      */
@@ -429,7 +431,7 @@ public abstract class XMLParser
      * @see #getValidation
      * @see #setFeature
      */
-    protected void setValidation(boolean validate) 
+    protected void setValidation(boolean validate)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             throw new SAXNotSupportedException("PAR004 Cannot setFeature(http://xml.org/sax/features/validation): parse is in progress.\n"+
@@ -451,7 +453,7 @@ public abstract class XMLParser
      *
      * @see #setValidation
      */
-    protected boolean getValidation() 
+    protected boolean getValidation()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fValidator.getValidationEnabled();
     }
@@ -492,7 +494,7 @@ public abstract class XMLParser
      *
      * @see #setExternalGeneralEntities
      */
-    protected boolean getExternalGeneralEntities() 
+    protected boolean getExternalGeneralEntities()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return true;
     }
@@ -533,7 +535,7 @@ public abstract class XMLParser
      *
      * @see #setExternalParameterEntities
      */
-    protected boolean getExternalParameterEntities() 
+    protected boolean getExternalParameterEntities()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return true;
     }
@@ -551,7 +553,7 @@ public abstract class XMLParser
      * @see #getNamespaces
      * @see #setFeature
      */
-    protected void setNamespaces(boolean process) 
+    protected void setNamespaces(boolean process)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             throw new SAXNotSupportedException("PAR004 Cannot setFeature(http://xml.org/sax/features/namespaces): parse is in progress.\n"+
@@ -568,7 +570,7 @@ public abstract class XMLParser
      *
      * @see #setNamespaces
      */
-    protected boolean getNamespaces() 
+    protected boolean getNamespaces()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fValidator.getNamespacesEnabled();
     }
@@ -588,7 +590,7 @@ public abstract class XMLParser
      * @see #getValidationSchema
      * @see #setFeature
      */
-    protected void setValidationSchema(boolean schema) 
+    protected void setValidationSchema(boolean schema)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             // REVISIT: Localize message
@@ -602,14 +604,14 @@ public abstract class XMLParser
      *
      * @see #setValidationSchema
      */
-    protected boolean getValidationSchema() 
+    protected boolean getValidationSchema()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fValidator.getSchemaValidationEnabled();
     }
 
     /**
-     * Allows the user to turn full Schema constraint checking on/off.  
-     * Only takes effect if Schema validation is enabled.                  
+     * Allows the user to turn full Schema constraint checking on/off.
+     * Only takes effect if Schema validation is enabled.
      * If this feature is off, partial constraint checking is done.
      * <p>
      * This method is equivalent to the feature:
@@ -622,7 +624,7 @@ public abstract class XMLParser
      * @see #getValidationSchemaFullChecking
      * @see #setFeature
      */
-    protected void setValidationSchemaFullChecking(boolean schemaFullChecking) 
+    protected void setValidationSchemaFullChecking(boolean schemaFullChecking)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             // REVISIT: Localize message
@@ -633,24 +635,39 @@ public abstract class XMLParser
 
 
     /**
-     * Allows the user to set a list of external XML Schemas (ex."http://example.com schema.xsd") 
+     * Normalization of element content is controlled by this feature.
+     * If this feature is set to true, DOM and SAX APIs will expose element content
+     * that is normalized according to XML Schema REC.
+     * Otherwise, element content will be exposed as required by the infoset.
+     *
+     * @param normalize
+     */
+    protected void setNormalizeContents(boolean normalize) {
+        fValidator.setNormalizeContents(normalize);
+    }
+    protected boolean getNormalizeContents() {
+        return fValidator.getNormalizeConents();
+    }
+
+    /**
+     * Allows the user to set a list of external XML Schemas (ex."http://example.com schema.xsd")
      * to be used by the parser.
      * If two schemas with the same targetNamespace appear in both
      * the list and a document, the one from the list will be picked up.
      * See XML Schema REC: http://www.w3.org/TR/xmlschema-1/#schema-loc
      * <P>
      * This method is equivalent to the property:
-     * <PRE>http://apache.org/xml/properties/validation/schema/external-schemaLocation</PRE>
-     * 
+     * <PRE>http://apache.org/xml/properties/schema/external-schemaLocation</PRE>
+     *
      * @param value  The list of schemas.
      * @exception SAXNotRecognizedException
      * @exception SAXNotSupportedException
      */
-    protected void setExternalSchemaLocation(Object value) 
+    protected void setExternalSchemaLocation(Object value)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             // REVISIT: Localize message
-            throw new SAXNotSupportedException("http://apache.org/xml/properties/validation/schema/external-schema-location:  parse is in progress");
+            throw new SAXNotSupportedException("http://apache.org/xml/properties/validation/schema/external-schemaLocation:  parse is in progress");
         }
         fValidator.setExternalSchemas(value);
     }
@@ -661,21 +678,21 @@ public abstract class XMLParser
      * See XML Schema REC: http://www.w3.org/TR/xmlschema-1/#schema-loc
      * <P>
      * This method is equivalent to the property:
-     * <PRE>http://apache.org/xml/properties/validation/schema/external-noNamespaceSchemaLocation</PRE>
-     * 
+     * <PRE>http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation</PRE>
+     *
      * @param value  An XML Schema file name
      * @exception SAXNotRecognizedException
      * @exception SAXNotSupportedException
      */
-    protected void setExternalNoNamespaceSchemaLocation(Object value) 
+    protected void setExternalNoNamespaceSchemaLocation(Object value)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             // REVISIT: Localize message
-            throw new SAXNotSupportedException("http://apache.org/xml/properties/validation/schema/external-noNamespaceSchemaLocation:  parse is in progress");
+            throw new SAXNotSupportedException("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation:  parse is in progress");
         }
         fValidator.setExternalNoNamespaceSchema(value);
     }
-    
+
     /**
      * Returns true if Schema support is turned on.
      *
@@ -703,7 +720,7 @@ public abstract class XMLParser
      * @see #getValidationDynamic
      * @see #setFeature
      */
-    protected void setValidationDynamic(boolean dynamic) 
+    protected void setValidationDynamic(boolean dynamic)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             // REVISIT: Localize message
@@ -723,7 +740,7 @@ public abstract class XMLParser
      *
      * @see #setValidationDynamic
      */
-    protected boolean getValidationDynamic() 
+    protected boolean getValidationDynamic()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fValidator.getDynamicValidationEnabled();
     }
@@ -736,7 +753,7 @@ public abstract class XMLParser
     }
 
     /**
-     * Allows the parser to have the choice to load DTD grammar when 
+     * Allows the parser to have the choice to load DTD grammar when
      * validation is off.
      * <p>
      * This method is the equivalent to the feature:
@@ -750,7 +767,7 @@ public abstract class XMLParser
      * @see #getLoadDTDGrammar
      * @see #setFeature
      */
-    protected void setLoadDTDGrammar(boolean loadDTDGrammar) 
+    protected void setLoadDTDGrammar(boolean loadDTDGrammar)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             // REVISIT: Localize message
@@ -769,13 +786,13 @@ public abstract class XMLParser
      *
      * @see #setLoadDTDGrammar
      */
-    protected boolean getLoadDTDGrammar() 
+    protected boolean getLoadDTDGrammar()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fValidator.getLoadDTDGrammar();
     }
 
     /**
-     * Allows the parser to have the choice to load the external DTD when 
+     * Allows the parser to have the choice to load the external DTD when
      * validation is off.
      * <p>
      * This method is the equivalent to the feature:
@@ -789,7 +806,7 @@ public abstract class XMLParser
      * @see #getLoadExternalDTD
      * @see #setFeature
      */
-    protected void setLoadExternalDTD(boolean loadExternalDTD) 
+    protected void setLoadExternalDTD(boolean loadExternalDTD)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         if (fParseInProgress) {
             // REVISIT: Localize message
@@ -808,7 +825,7 @@ public abstract class XMLParser
      *
      * @see #setLoadExternalDTD
      */
-    protected boolean getLoadExternalDTD() 
+    protected boolean getLoadExternalDTD()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fScanner.getLoadExternalDTD();
     }
@@ -888,7 +905,7 @@ public abstract class XMLParser
      * @see #getAllowJavaEncodings
      * @see #setFeature
      */
-    protected void setAllowJavaEncodings(boolean allow) 
+    protected void setAllowJavaEncodings(boolean allow)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         fEntityHandler.setAllowJavaEncodings(allow);
     }
@@ -898,7 +915,7 @@ public abstract class XMLParser
      *
      * @see #setAllowJavaEncodings
      */
-    protected boolean getAllowJavaEncodings() 
+    protected boolean getAllowJavaEncodings()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fEntityHandler.getAllowJavaEncodings();
     }
@@ -928,7 +945,7 @@ public abstract class XMLParser
      *
      * @see #setContinueAfterFatalError
      */
-    protected boolean getContinueAfterFatalError() 
+    protected boolean getContinueAfterFatalError()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fContinueAfterFatalError;
     }
@@ -955,7 +972,7 @@ public abstract class XMLParser
      * @see #setProperty
      */
     /***
-    protected void setNamespaceSep(String separator) 
+    protected void setNamespaceSep(String separator)
         throws SAXNotRecognizedException, SAXNotSupportedException {
         // REVISIT: Ask someone what it could possibly hurt to allow
         //          the application to change this in mid-parse.
@@ -972,7 +989,7 @@ public abstract class XMLParser
      * @see #setNamespaceSep
      */
     /***
-    protected String getNamespaceSep() 
+    protected String getNamespaceSep()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         return fNamespaceSep;
     }
@@ -986,7 +1003,7 @@ public abstract class XMLParser
      *
      * @see #getProperty
      */
-    protected String getXMLString() 
+    protected String getXMLString()
         throws SAXNotRecognizedException, SAXNotSupportedException {
         throw new SAXNotSupportedException("http://xml.org/sax/properties/xml-string");
     }
@@ -1343,6 +1360,11 @@ public abstract class XMLParser
                 // REVISIT
                 throw new SAXNotSupportedException(featureId);
             }
+
+            if (feature.equals("schema/expose-normalized-values")) {
+                setNormalizeContents(state);
+                return;
+             }
             //
             // http://apache.org/xml/features/validation/normalize-attribute-values
             //
@@ -1441,7 +1463,7 @@ public abstract class XMLParser
      * @exception org.xml.sax.SAXException If there is any other
      *            problem fulfilling the request.
      */
-    public boolean getFeature(String featureId) 
+    public boolean getFeature(String featureId)
         throws SAXNotRecognizedException, SAXNotSupportedException {
 
         //
@@ -1496,8 +1518,15 @@ public abstract class XMLParser
             //
             if (feature.equals("validation/schema")) {
                 return getValidationSchema();
+            } 
+            // http://apache.org/xml/features/validation/schema-full-checking
+            //   Lets the user turn Schema full constraint checking on/off.
+            //
+            if (feature.equals("validation/schema-full-checking")) {
+                return getValidationSchemaFullChecking();
             }
             //
+            //
             // http://apache.org/xml/features/validation/dynamic
             //   Allows the parser to validate a document only when it
             //   contains a grammar. Validation is turned on/off based
@@ -1513,6 +1542,9 @@ public abstract class XMLParser
                 // REVISIT
                 throw new SAXNotRecognizedException(featureId);
             }
+            if (feature.equals("validation/normalize-element-contents")) {
+                return getNormalizeContents();
+            }
             //
             // http://apache.org/xml/features/validation/validate-content-models
             //
@@ -1611,7 +1643,7 @@ public abstract class XMLParser
         String property;
         if (propertyId.startsWith(SAX2_PROPERTIES_PREFIX)) {
             property = propertyId.substring(SAX2_PROPERTIES_PREFIX.length());
-                                                
+
             //
             // http://xml.org/sax/properties/namespace-sep
             // Value type: String
@@ -1633,7 +1665,7 @@ public abstract class XMLParser
                 return;
             }
             /***/
-            
+
             //
             // http://xml.org/sax/properties/xml-string
             // Value type: String
@@ -1658,10 +1690,10 @@ public abstract class XMLParser
         // Xerces Properties
         //
 
-        
+
         if (propertyId.startsWith(XERCES_PROPERTIES_PREFIX)) {
             property = propertyId.substring(XERCES_PROPERTIES_PREFIX.length());
-            
+
             if (property.equals("schema/external-schemaLocation")) {
                 setExternalSchemaLocation(value);
                 return;
@@ -1670,10 +1702,10 @@ public abstract class XMLParser
                 setExternalNoNamespaceSchemaLocation(value);
                 return;
             }
-                
+
 
         }
-        
+
 
         //
         // Not recognized
@@ -1698,7 +1730,7 @@ public abstract class XMLParser
      *            problem fulfilling the request.
      * @see org.xml.sax.XMLReader#getProperty
      */
-    public Object getProperty(String propertyId) 
+    public Object getProperty(String propertyId)
         throws SAXNotRecognizedException, SAXNotSupportedException {
 
         //
@@ -1743,7 +1775,7 @@ public abstract class XMLParser
         // Xerces Properties
         //
 
-       
+
         else if (propertyId.startsWith(XERCES_PROPERTIES_PREFIX)) {
             //
             // No properties defined yet that are common to all parsers.
@@ -1761,7 +1793,7 @@ public abstract class XMLParser
                 return fValidator.getExternalNoNamespaceSchema();
             }
         }
-       
+
 
         //
         // Not recognized
