@@ -79,7 +79,7 @@ import org.xml.sax.helpers.LocatorImpl;
 /**
  * Default entity handler implementation.
  *
- * @version $Id: DefaultEntityHandler.java,v 1.8 2001/07/20 17:26:26 lmartin Exp $
+ * @version $Id: DefaultEntityHandler.java,v 1.8.2.2 2001/11/05 19:48:21 neilg Exp $
  */
 public class DefaultEntityHandler 
     implements XMLEntityHandler, XMLEntityHandler.DTDHandler {
@@ -391,7 +391,9 @@ public class DefaultEntityHandler
 
         // check for bad parameters id
         if (id == null || id.length() == 0) {
-            return systemId;
+            // REVISIT: returning null here will result in NPE
+            // for notations and entity declarations
+            return "";
         }
 
         // if id already expanded, return
@@ -502,7 +504,9 @@ public class DefaultEntityHandler
                                         0,
                                         args,
                                         XMLErrorReporter.ERRORTYPE_FATAL_ERROR);
-        } catch (FileNotFoundException fnf) {
+        } catch (FileNotFoundException fnf) { 
+            // according to the JAXP spec, SAXExceptions should not be 
+            // generated in this case!  - neilg
             String errorSystemId = fSystemId;
             fEventHandler.endEntityReference(fEntityName, fEntityType, fEntityContext);
             popReader();
@@ -514,7 +518,8 @@ public class DefaultEntityHandler
                                         ImplementationMessages.IO0,
                                         0,
                                         args,
-                                        XMLErrorReporter.ERRORTYPE_FATAL_ERROR);
+                                        XMLErrorReporter.ERRORTYPE_RECOVERABLE_ERROR);
+            throw fnf;
         } catch (UnsupportedEncodingException uee) {
             fEventHandler.endEntityReference(fEntityName, fEntityType, fEntityContext);
             popReader();
