@@ -2,7 +2,7 @@
  * The Apache Software License, Version 1.1
  *
  *
- * Copyright (c) 1999,2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999,2001 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -88,7 +88,7 @@ import java.io.Serializable;
 * default port for a specific scheme). Rather, it only knows the
 * grammar and basic set of operations that can be applied to a URI.
 *
-* @version  $Id: URI.java,v 1.8 2001/07/13 17:54:04 sandygao Exp $
+* @version  $Id: URI.java,v 1.8.2.1 2001/11/12 20:24:15 sandygao Exp $
 *
 **********************************************************************/
  public class URI implements Serializable {
@@ -377,15 +377,21 @@ import java.io.Serializable;
     int uriSpecLen = uriSpec.length();
     int index = 0;
 
-    // Check for scheme, which must be before `/'. Also handle names with
-    // DOS drive letters ('D:'), so 1-character schemes are not allowed.
-    int colonIdx = uriSpec.indexOf(':');
-    int slashIdx = uriSpec.indexOf('/');
-    if ((colonIdx < 2) || (colonIdx > slashIdx && slashIdx != -1)) {
-      int fragmentIdx = uriSpec.indexOf('#');
+    // Check for scheme, which must be before '/', '?' or '#'. Also handle
+    // names with DOS drive letters ('D:'), so 1-character schemes are not
+    // allowed.
+    int colonIdx    = uriSpec.indexOf(':');
+    int slashIdx    = uriSpec.indexOf('/');
+    int queryIdx    = uriSpec.indexOf('?');
+    int fragmentIdx = uriSpec.indexOf('#');
+
+    if ((colonIdx < 2) ||
+        (colonIdx > slashIdx && slashIdx != -1) ||
+        (colonIdx > queryIdx && queryIdx != -1) ||
+        (colonIdx > fragmentIdx && fragmentIdx != -1)) {
       // A standalone base is a valid URI according to spec
-      if (p_base == null && fragmentIdx != 0 ) {
-        throw new MalformedURIException("No scheme found in URI.");
+      if (p_base == null && fragmentIdx != 0) {
+              throw new MalformedURIException("No scheme found in URI.");
       }
     }
     else {
