@@ -125,7 +125,7 @@ import org.xml.sax.SAXException;
  * </ul>
  *
  *
- * @version $Revision: 1.15 $ $Date: 2001/04/18 06:02:49 $
+ * @version $Revision: 1.15.2.1 $ $Date: 2001/11/07 19:00:34 $
  * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
  * @see Serializer
  */
@@ -314,7 +314,10 @@ public class HTMLSerializer
                     } else {
                         // HTML: Empty values print as attribute name, no value.
                         // HTML: URI attributes will print unescaped
-                        if ( value == null || value.length() == 0 )
+                        if ( value == null ) {
+                            value = "";
+                        }
+                        if ( !_format.getPreserveEmptyAttributes() && value.length() == 0 )
                             _printer.printText( name );
                         else if ( HTMLdtd.isURI( rawName, name ) ) {
                             _printer.printText( name );
@@ -558,7 +561,10 @@ public class HTMLSerializer
                     } else {
                         // HTML: Empty values print as attribute name, no value.
                         // HTML: URI attributes will print unescaped
-                        if ( value == null || value.length() == 0 )
+                        if ( value == null ) {
+                            value = "";
+                        }
+                        if ( !_format.getPreserveEmptyAttributes() && value.length() == 0 )
                             _printer.printText( name );
                         else if ( HTMLdtd.isURI( tagName, name ) ) {
                             _printer.printText( name );
@@ -768,7 +774,10 @@ public class HTMLSerializer
                     } else {
                         // HTML: Empty values print as attribute name, no value.
                         // HTML: URI attributes will print unescaped
-                        if ( value == null || value.length() == 0 )
+                        if ( value == null ) {
+                            value = "";
+                        }
+                        if ( !_format.getPreserveEmptyAttributes() && value.length() == 0 )
                             _printer.printText( name );
                         else if ( HTMLdtd.isURI( tagName, name ) ) {
                             _printer.printText( name );
